#ifndef ds50daq_DAQ_Aggregator_hh
#define ds50daq_DAQ_Aggregator_hh

#include <string>
#include <vector>

#include "fhiclcpp/ParameterSet.h"
#include "art/Persistency/Provenance/RunID.h"
#include "artdaq/DAQrate/quiet_mpi.hh"
#include "artdaq/DAQdata/FragmentGenerator.hh"
#include "artdaq/DAQrate/RHandles.hh"
#include "artdaq/DAQrate/GlobalQueue.hh"
#include "artdaq/DAQrate/EventStore.hh"

namespace ds50
{
  class Aggregator;
}

class ds50::Aggregator
{
public:
  Aggregator(int mpi_rank);
  Aggregator(Aggregator const&) = delete;
  ~Aggregator();
  Aggregator& operator=(Aggregator const&) = delete;

  bool initialize(fhicl::ParameterSet const&);
  bool start(art::RunID);
  bool stop();
  bool pause();
  bool resume();
  bool shutdown();
  bool soft_initialize(fhicl::ParameterSet const&);
  bool reinitialize(fhicl::ParameterSet const&);

  size_t process_fragments();

  std::string report(std::string const&) const;

private:
  int mpi_rank_;
  art::RunID run_id_;
  bool local_group_defined_;
  bool art_initialized_;
  MPI_Comm local_group_comm_;

  std::string init_string_;
  uint64_t max_fragment_size_words_;
  size_t mpi_buffer_count_;
  size_t first_data_sender_rank_;
  size_t data_sender_count_;
  size_t expected_events_per_bunch_;
  bool print_event_store_stats_;

  std::unique_ptr<artdaq::RHandles> receiver_ptr_;
  std::unique_ptr<artdaq::EventStore> event_store_ptr_;
  artdaq::RawEventQueue &event_queue_;
  fhicl::ParameterSet previous_pset_;
};

#endif

//  LocalWords:  ds
